//
// CActiveConnect
//

// A CActive derived Connection class for Internet sockets

#ifndef __ACTIVE_CONNECT_H__
#define __ACTIVE_CONNECT_H__

#include <es_sock.h>

class MConnectionObserver;

class CActiveConnect : public CActive {
public:
    
    static CActiveConnect* NewL( MConnectionObserver* aObserver
		               , const TProtocolName& aName
			       , RSocket&             aSocket
			       );
    static CActiveConnect* NewLC( MConnectionObserver* aObserver
	                        , const TProtocolName& aName
				, RSocket&             aSocket
				);
    virtual ~CActiveConnect();


    enum TDisconnectType {
	EDisconnectSocket
      , EDisconnectSocketAndNet
    };

    void Connect(const THostName& aPeerName, TUint aPort);
    void Disconnect(TDisconnectType t = EDisconnectSocket);

    TBool IsActiveConnection() const { return iPhase != EDisconnected; }
    TBool IsConnected() const { return iPhase == EConnected; }

    // Panics and Leaves
    enum Panics {
	EWrongPhase
    };

private:

    enum TPhase {
        EDisconnected
      ,	EStartingServer
      , EResolvingAddress
      , EConnecting
      , EConnected
      , EDisconnecting
    };

    RSocketServ   iCommServer;
    TProtocolDesc iProtocol;
    RSocket&      iSocket;
    THostName     iPeerName;
    TUint         iPeerPort;
    RHostResolver iAddrDB;
    TNameEntry    iResolvedAddr;
    MConnectionObserver* iObserver;
    TPhase          iPhase;
    TDisconnectType iDisconnectType;

    void DoCancel();
    void RunL();

    void StartProtocol();
    void ResolveAddress();
    void CompleteConnectL();
    void OnSocketConnect();
    void CloseConnection();
    void DisconnectNetwork();

    CActiveConnect(RSocket& aSocket);
    void ConstructL( const TProtocolName& aName
		   , MConnectionObserver* aObserver
		   );

};


#endif // __ACTIVE_CONNECT_H__