// NVT.CPP
//
//
#include "nvt.h"
#include <eikenv.h>

#include "ActiveConnect.h"
#include "ActiveConnection.h"
#include "NVTObserver.h"

#define TCP_NAME _L("tcp")
#define NVT_CATEGORY _L("NVT")


void MNVTObserver::OnBufferUpdateL(const TDesC& aBuf) {
    for (TInt i = 0; i < aBuf.Length(); ++i) {
	OnBufferUpdateL(aBuf[i]);
    }
}


CNVT* CNVT::NewLC(MNVTObserver* aObserver) {
    CNVT* self = new (ELeave) CNVT();
    CleanupStack::PushL(self);
    
    self->ConstructL(aObserver);

    return self;
}

CNVT* CNVT::NewL(MNVTObserver* aObserver) {
    CNVT* self = NewLC(aObserver);
    CleanupStack::Pop();
    return self;
}


CNVT::CNVT()
: iInputMode(Data)
, iRequest(NULL)
, iRead(NULL)
, iWrite(NULL)
, iObserver(NULL)
, iLocalEcho(ETrue) 
, iSuppressGA(EFalse) {
}


CNVT::~CNVT() {

    if (iRequest->IsActiveConnection()) {
//    	User::Panic(NVT_CATEGORY, KErrInUse);
    }

    delete iRequest;
    delete iRead;
    delete iWrite;
}


void CNVT::ConstructL(MNVTObserver* aObserver) {

    iObserver = aObserver;

    iRequest = CActiveConnect::NewL(this, TCP_NAME, iConnection);
}


void CNVT::WriteL(const TChar aChar) {
    if (IsConnected()) {
	TRequestStatus status;

	TBuf8<4> buffer(1);
	buffer[0] = TUint8(aChar);
	if (buffer[0] == 0x0D) {
//	    buffer.SetLength(4);
	    buffer[0] = 10;
//	    buffer[2] = IAC;
//	    buffer[3] = DataMark;
	}
	else if (buffer[0] == 8) {
	    buffer.SetLength(2);
	    buffer[0] = IAC;
	    buffer[1] = EraseCharacter;
	}

	iWrite->WriteL(buffer);

	if (iObserver && iLocalEcho) {
	    iObserver->OnBufferUpdateL(aChar);
	}
    }
}


void CNVT::OnReceiveL(const TPtrC8 aBuffer) {

    TInt scan_start = 0;
    for (TInt i = 0; i < aBuffer.Length(); ++i) {

	scan_start = i;

	switch(iInputMode) {

	case Command:
	    OnCommandL(aBuffer[i]);
	    break;

	case Data:

	    if ((i = aBuffer.Mid(scan_start).Locate(IAC)) != KErrNotFound) {
		i += scan_start;
		iInputMode = Command;
		if (   iObserver
		    && i - scan_start
		    ) {
		    iObserver->OnBufferUpdateL(aBuffer.Mid(scan_start, i - scan_start));
		}
	    }
	    else {
		i = aBuffer.Length();
		if (iObserver) {
		    iObserver->OnBufferUpdateL(aBuffer.Mid(scan_start));
		}
	    }
	    break;
	}
    }
}

void CNVT::OnCommandL(TUint8 aCommand) {

    if (iCommandState == OptionPending) {
	iInputMode = Data;
	iCommandState = Initial;
	OnOptionCommandL(TCommands(iPrevByte), aCommand);
	return;
    }
    
    switch (aCommand) {
    case DONT:
    case DO:
    case WONT:
    case WILL:
	iCommandState = OptionPending;
	iPrevByte     = aCommand;
	break;
    case IAC:
    case GoAhead:
    case EraseLine:
    case EraseCharacter:
    case AreYouThere:
    case AbortOutput:
    case InterruptProcess:
    case Break:
    case DataMark:
    case NOP:
    case SB:
    case SE:
    default:
	iInputMode = Data;
	iCommandState = Initial;
	break;
    }
}

void CNVT::OnOptionCommandL(TCommands aCommand, TUint8 aOption) {
#if 0
    if (aOption == Echo) {
	switch (aCommand) {
	case DONT:
	    ReplyOption(WONT, aOption);
	    iLocalEcho = ETrue;
	    break;

	case DO:
	    ReplyOption(WILL, aOption);
	    iLocalEcho = EFalse;
	    break;

	case WONT:
	    ReplyOption(DONT, aOption);
	    iLocalEcho = ETrue;
	    break;

	case WILL:

	    ReplyOption(DO, aOption);
	    iLocalEcho = EFalse;
	    break;
	}
    }
    else if (aOption == SuppressGA) {
 	switch (aCommand) {
	case DONT:
	    ReplyOption(WONT, aOption);
	    iSuppressGA = EFalse;
	    break;

	case DO:
	    ReplyOption(WILL, aOption);
	    iSuppressGA = ETrue;
	    break;

	case WONT:
	    ReplyOption(DONT, aOption);
	    iSuppressGA = EFalse;
	    break;

	case WILL:

	    ReplyOption(DO, aOption);
	    iSuppressGA = ETrue;
	    break;
	}
   }else
#endif
     {


	switch (aCommand) {
	case DONT:
	    ReplyOptionL(WONT, aOption);
	    break;

	case DO:
	    ReplyOptionL(WONT, aOption);
	    break;

	case WONT:
	    ReplyOptionL(DONT, aOption);
	    break;

	case WILL:

	    ReplyOptionL(DONT, aOption);
	    break;
	}
    }
}


void CNVT::ReplyOptionL(TCommands aCommand, TUint8 aOption) {

    TBuf8<3> command(3);
    command[0] = IAC;
    command[1] = TUint8(aCommand);
    command[2] = aOption;

    iWrite->WriteL(command);

}

TBool CNVT::IsConnected() const {
    return iRequest->IsConnected();
}



void CNVT::ConnectL(const TDesC& aAddr, const TUint aPort) {

    DisconnectL();

    if (iRead || iWrite) {
	User::Panic(NVT_CATEGORY, KErrAlreadyExists);
    }
    iRead  = CSocketRead::NewLC(iConnection, this);
    iWrite = CSocketWrite::NewLC(iConnection);

    iRequest->Connect(aAddr, aPort);

    CleanupStack::Pop(2);
}

void CNVT::DisconnectL(TDisconnectType aType) {

    delete iRead;
    iRead = NULL;

    delete iWrite;
    iWrite = NULL;

    CActiveConnect::TDisconnectType type = aType == EDisconnectNormal ? CActiveConnect::EDisconnectSocket : CActiveConnect::EDisconnectSocketAndNet;
    iRequest->Disconnect(type);
}


// return ETrue to proceed to the next phase
void CNVT::OnServerStart(TRequestStatus aStatus) {

    if (iObserver) {
	iObserver->OnServerStart(aStatus);
    }
}

void CNVT::OnAddressResolved(TRequestStatus aStatus) {
    if (iObserver) {
	iObserver->OnAddressResolved(aStatus);
    }
}

void CNVT::OnConnect(TRequestStatus aStatus) {

    if (iObserver) {
	iObserver->OnConnect(aStatus);
    }

    if (aStatus == KErrNone) {

	iInputMode   = Data;
	iRead->Receive();
    }
}

void CNVT::OnDisconnect(TRequestStatus aStatus) {
    if (iObserver) {
	iObserver->OnDisconnect(aStatus);
    }
}


TBool CNVT::OnReadErrorL(TRequestStatus aStatus) {
    switch (aStatus.Int()) {
    
    case KErrEof:   // orderly shutdown
	// fall through

    default:	    // some other error.
	DisconnectL();
	return EFalse;
    }
}


