// NVT.H
//
// Copyright (c) 1997 Psion Software plc.  All rights reserved.
//

#if !defined(__NVT_H__)
#define __NVT_H__

#include <es_sock.h>

#include "ACObservers.h"

class NVTConnection;
class CGlobalText;
class CParaFormatLayer;
class CCharFormatLayer;
class CSocketRead;
class CActiveConnect;
class CSocketWrite;
class MSockReadObserver;
class MNVTObserver;

class CNVT : public CBase 
	   , public MConnectionObserver
	   , public MSockReadObserver {
public:
    enum {
	EDefaultPort = 23
    };

    static CNVT* NewLC(MNVTObserver* aObserver);
    static CNVT* NewL(MNVTObserver* aObserver);
    virtual ~CNVT();

    enum TDisconnectType {
	EDisconnectNormal
      , EDisconnectNet
    };


    void ConnectL(const TDesC& aAddr, const TUint aPort);
    void DisconnectL(TDisconnectType aType = EDisconnectNormal);

//    void      DoL();
//    void      WillL();
//    void      DontL();
//    void      WontL();
    void      WriteL(const TChar aChar);

    TBool     IsConnected() const;

private:

    // -- MConnectionObserver
    virtual void OnServerStart(TRequestStatus aStatus);
    virtual void OnAddressResolved(TRequestStatus aStatus);
    virtual void OnConnect(TRequestStatus aStatus);
    virtual void OnDisconnect(TRequestStatus aStatus);

private:
    virtual void  OnReceiveL(const TPtrC8);
    virtual TBool OnReadErrorL(TRequestStatus aStatus);

private:

    CNVT();
    void ConstructL(MNVTObserver* aObserver);


    enum TCommands {
	IAC              = 255
      , DONT             = 254 
      , DO               = 253
      , WONT             = 252
      , WILL             = 251
      , SB               = 250
      , GoAhead          = 249
      , EraseLine        = 248
      , EraseCharacter   = 247
      , AreYouThere      = 246
      , AbortOutput      = 245
      , InterruptProcess = 244
      , Break            = 243
      , DataMark         = 242
      , NOP              = 241
      , SE               = 240
    };

    enum StreamMode {
	Data
      , Command
    };

    enum CommandState {
	Initial
      , OptionPending
    };

    CommandState iCommandState;
    TUint8       iPrevByte;

    void OnCommandL(TUint8 aCommand);
    void OnOptionCommandL(TCommands aCommand, TUint8 aOption);
    void ReplyOptionL(TCommands aCommand, TUint8 aOption);

    StreamMode    iInputMode;

    RSocket       iConnection;
    CActiveConnect* iRequest;
    CSocketRead*    iRead;
    CSocketWrite*   iWrite;

    MNVTObserver*   iObserver;

    enum TKnownOptions {
	TransmitBinary = 0
      ,	Echo           = 1
      , SuppressGA     = 3
      , Status         = 5
      , TimingMark     = 6
      , ExtendedOption = 255
    };

    TBool           iLocalEcho;
    TBool           iSuppressGA;
};



#endif